<?php   

include("global.inc.php");

function removecrlf($string)
{
    return strtr($string, "\015\012", "  ");
}


/*
    Encode header according to rfc2047. Used for personal parts of mail 
    addresses and subject.
*/
function mime_header_encode($string, $charset = 'UTF-8')
{
    $numtoenc = preg_match_all('/[^\x20-\x7E]/', $string, $matches);
    $len = strlen($string);
    if ($numtoenc > 0)
    { 
        /* Find smallest encoding: Q ads $numtoenc * 2, B means *8/6 */
        if ($len + ($numtoenc * 2) > (($len * 8) / 6))
        {
            $encoding = 'B';
            $encoded = base64_encode($string);
        }
        else
        {
            $encoding = 'Q';
            $encoded = imap_8bit($string);
            /* imap_8bit does 75-char-line-split already; undo */
            $encoded = str_replace("=\r\n", "", $encoded);
            $encoded = str_replace("\r\n", "", $encoded);
            /* Now also encode spaces as specified */
            $encoded = str_replace(" ", "_", $encoded);
        }

        $chunk_size = 75 - 7 - strlen($charset); 
        $chunk_size -= $chunk_size % 4; 
        $encoded = trim(chunk_split($encoded, $chunk_size)); 
        $lines = explode("\r\n", $encoded);
        for ($i = 0; $i < count($lines); $i++)
        {
            $lines[$i] = preg_replace('/^(.*)$/m', "=?$charset?$encoding?\\1?=", $lines[$i]); 
        }
        $encoded = implode("\r\n ", $lines);
    } 
    else
    {
        $encoded = $string;
    }
    return $encoded; 
}


function cleanaddress($adr, $domain)
{
    $a = stripslashes($adr);
    $a = removecrlf($a);
    $alist = imap_rfc822_parse_adrlist($a, $domain);
    for ($i = 0; $i < count($alist); $i++)
    {
        $alist[$i]->personal = mime_header_encode($alist[$i]->personal);
    }
    return make_address_list($alist);
}

function mailfrom($fromaddress, $body, $headers)
{
    // -t means get To, CC and BCC from message
    // -i means ignore single dots on line, read till EOF
    // -f forces fromaddress
    $fp = popen("/usr/sbin/sendmail -t -i -f" . $fromaddress, "w");

    fputs($fp, $headers);
    fputs($fp, $body);
    fputs($fp, CRLF);

    pclose($fp);
}

/* A boundary must be unlikely to appear in the body. "=_" can never
   occur in qprint or bas64 encoded parts. The user could still type
   this, but '=' is converted in qprint and will never appear
   in bas64 except in the end.
*/
$boundary = "=_Message-Boundary_=";
$mailheaders = "";

if (SERVER_SUFFIX)
    $domain = SERVER_SUFFIX;
else
    $domain = IMAP_SERVER;

if (FORCE_FROM and strpos($user, '@') === FALSE)
    $from_user = "$user@$domain";
else
    $from_user = $user;

if (FORCE_FROM)
    $mailheaders .= "From: $from_user" . CRLF;
$to = $_POST['to'];
if ($to != "")
    $mailheaders .= "To: ". cleanaddress($to, $domain) . CRLF;
$cc = $_POST['cc'];
if ($cc != "")
    $mailheaders .= "Cc: " . cleanaddress($cc, $domain) . CRLF;
$bcc = $_POST['bcc'];
if ($bcc != "")
    $mailheaders .= "Bcc: " . cleanaddress($bcc, $domain) . CRLF;

// Do subject earlier to accomodate buggy Mozilla mail summary box,
// according to Laird Bedore (lmbedore@vectorstar.com) 
$subject = $_POST['subject'];
$mailheaders .= "Subject: " . mime_header_encode(removecrlf(stripslashes($subject))) . CRLF;
$headers = $_POST['headers'];
$mailheaders .= $headers; // Additional headers from compose
$mailheaders .= "X-Mailer: " . VERSION . CRLF;

$body = $_POST['body'];
$msg_body = stripslashes($body);
$msg_body = imap_8bit($msg_body);
// 'Fix' crlfs in body to prevent double spacing
$msg_body = implode(CRLF, explode("\r\n", $msg_body));

$attach_tmp_name = $_FILES['attach']['tmp_name'];
$attach_size = $_FILES['attach']['size'];
$attach_name = $_FILES['attach']['name'];
$attach_type = $_FILES['attach']['type'];
if (( $attach_name != "none" ) && ( $attach_name != "" ) and (is_uploaded_file($attach_tmp_name)))
{
    $file = fopen($attach_tmp_name, "r");
    $contents = fread($file, $attach_size);
    $encoded_attach = chunk_split(base64_encode($contents));
    fclose($file);
    
    $mailheaders .= "MIME-version: 1.0" . CRLF;
    $mailheaders .= "Content-type: multipart/mixed; ";
    $mailheaders .= "boundary=\"$boundary\"" . CRLF;
    $mailheaders .= "Content-transfer-encoding: 7BIT" . CRLF;
    $mailheaders .= "X-attachments: $attach_name" . CRLF;

    $body_top = "--$boundary" . CRLF;
    $body_top .= "Content-type: text/plain; charset=UTF-8" . CRLF;
    $body_top .= "Content-transfer-encoding: quoted-printable" . CRLF;
    $body_top .= "Content-description: Mail message body" . CRLF . CRLF;

    $msg_body = $body_top . $msg_body;

    $msg_body .= CRLF . CRLF . "--$boundary" . CRLF;
    $msg_body .= "Content-type: $attach_type; name=\"$attach_name\"" . CRLF;       
    $msg_body .= "Content-Transfer-Encoding: BASE64" . CRLF;
    $msg_body .= "Content-disposition: attachment; filename=\"$attach_name\"" . CRLF . CRLF;
    $msg_body .= "$encoded_attach" . CRLF;
    $msg_body .= "--$boundary--" . CRLF;
}
else
{
    $mailheaders .= "MIME-version: 1.0" . CRLF;
    $mailheaders .= "Content-type: text/plain; charset=UTF-8" . CRLF;
    $mailheaders .= "Content-transfer-encoding: quoted-printable" . CRLF;
}

$mailheaders .= CRLF;

mailfrom($from_user, $msg_body, $mailheaders);
$msgnum = $_POST['msgnum'];
if ($msgnum != "")
{
    imap_setflag_full($mailbox, $msgnum, "\\Answered", ST_UID);
}

// add message to the sent mail folder
if (SENT_MAIL)
{
    $mailbox = mailbox_log_in($user, $pass, SENT_MAIL);
    $snt_folder = construct_folder_str(SENT_MAIL);

    if(imap_last_error())
    {
        //if there was an error, it was because there was no sent mail folder
        //we'll just create one here, then re-login

        imap_createmailbox($mailbox, IMAP_STR . "$snt_folder");
        $mailbox = mailbox_log_in($user, $pass, SENT_MAIL);
    }

    imap_append($mailbox, IMAP_STR. "$snt_folder", $mailheaders . $msg_body);
}

$msglist = $_POST['msglist'];
if (is_array($msglist))
{
    $url = "message.php?folder=" . urlencode($folder);
    foreach ($msglist as $m)   
    {
        $url .= "&msglist[]=$m";
    }
    if ($msgnum != "")
    {
        $url .= "#$msgnum";
    }
    header("Location: $url");  
}
else
{
    header("Location: index.php?folder=" . $folder);  
}
?>
